@@ -50,7 +50,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 2009-2011 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009-2015 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
@@ -86,7 +86,7 @@
  * only if the new code is made subject to such option by the copyright
  * holder.
  *
- * Portions Copyright 2012 OPEN-XCHANGE, licensed under GPL Version 2.
+ * Portions Copyright 2016-2020 OX Software GmbH, licensed under GPL Version 2.
  */
 package com.openexchange.http.grizzly.service.http;
 
@@ -108,7 +108,7 @@ public interface OSGiHandler {
      *
      * @return Processing lock.
      */
-    public ReentrantReadWriteLock.ReadLock getProcessingLock();
+    ReentrantReadWriteLock.ReadLock getProcessingLock();
 
     /**
      * Removal lock.
@@ -119,5 +119,5 @@ public interface OSGiHandler {
      * @see #getProcessingLock()
      * @see ReentrantReadWriteLock.WriteLock
      */
-    public ReentrantReadWriteLock.WriteLock getRemovalLock();
+    ReentrantReadWriteLock.WriteLock getRemovalLock();
 }
